// #include <stdio.h>
// #include <termio.h>
// #include <unistd.h>
// #include <dynamixel.h>
// #include "generalDefinitions.h"

#include "dxConsole.h"

DxComm dxComm = DxComm(new DevCommUsb());

int main()
{
	int option = -1;
	bool terminate=false;
	chosenID = -1;
	chosenDevice = 0;
	
	while (!terminate)
	{
		option = mainMenu();
		switch (option)
		{
			case 1:
				searchAndList();
				break;
			case 2:
				setChosenID();
				break;
			case 3:
				if (chosenDevice != AXS1 && chosenDevice != IMU6AXIS)
					printMotorData();
				else if (chosenDevice == IMU6AXIS)
					printImuData();
				else if (chosenDevice == AXS1)
					printAXs1Data();
				break;
			case 4:
				if ( chosenDevice != AXS1)
					toggleLed();
				break;
			case 5:
				if (chosenDevice != AXS1 && chosenDevice != IMU6AXIS)
					motorToPos();
				break;
			case 6:
				if (chosenDevice != AXS1 && chosenDevice != IMU6AXIS)
					motorAngleLimits();
				break;
			case 7:
				if (chosenDevice != AXS1 && chosenDevice != IMU6AXIS)
					motorTorqueLimit();
				break;
			case 8:
				if (chosenDevice != AXS1 && chosenDevice != IMU6AXIS)
					motorDefaultValues();
				break;
			case 98:
				setID();
				break;
//			case 97:
//				setInterfaceBaud();
//				break;
//			case 99:
//				setBaud();
//				break;
			case 0:
				terminate=true;
				break;
			default:
				printf("Unknown option!\n");
		}
		if (!terminate)
			backToMainMenu();
	}

	return 0;
}


int mainMenu()
{
	int ret, opt;
	printf("\n----------DETI dynamixel Console interface----------\n");
	printf(" 1 - List devices.\n");
	printf(" 2 - Choose ID for operations\n");

	switch (chosenDevice)
	{
		case AX12:
		case AX18:
		case RX28:
		case RX64:
		case MX106:
			printf(" 3 - Print control table.\n");
			printf(" 4 - Toggle LED.\n");
			printf(" 5 - Goto position.\n");
			printf(" 6 - Define position limits.\n");
			printf(" 7 - Define max torque.\n");
			printf(" 8 - Reset to default values.\n");
			break;
		case AXS1:
			printf(" 3 - Print control table.\n");
			break;
		case IMU6AXIS:
			printf(" 3 - Print control table.\n");
			printf(" 4 - Toggle LED.\n");
			break;
		default:
			break;
	}
	printf(" 98 - Set ID (ATTENTION: operation on all connected devices!).\n");
	printf(" 0 - Terminate application\n");
	
	printf("CurrentID: %d - ",chosenID);
	if (chosenDevice != AXS1 && chosenDevice != IMU6AXIS)
		printf("Servo motor");
	else if (chosenDevice == IMU6AXIS)
		printf("6 Axis inertial unit");
	else if (chosenDevice == AXS1)
		printf("AX-S1 sensor module");
	printf("\n");
	
	printf("Choose your option: ");
	ret = scanf("%d",&opt);
	printf("\n");
	
	return opt;
}


void searchAndList()
{
	printf("Searching for devices...\n");
	int model, series;

	series = AX_SERIES;

	for (unsigned int i=0; i<=MAX_DYNAMIXEL; i++)
	{
		//if a device with the current id is present, pool for its model number and display information accordingly
		if (dxComm.dxl_ping(series, i))
		{
			model=dxComm.dxl_read_word(series,i,P_MODEL_NUMBER_L);
			if (model==AX12)
				printf("ID: %3d - AX-12 servo\n",i);
			else if (model==IMU6AXIS)
				printf("ID: %3d - 6 Axis inertial unit\n",i);
			else if (model==AXS1)
				printf("ID: %3d - AX-S1 sensor module\n",i);
			else if (model==AX18)
				printf("ID: %3d - AX-18 servo\n",i);
			else if (model==RX28)
				printf("ID: %3d - RX-28 servo\n",i);
			else if (model==RX64)
				printf("ID: %3d - RX-64 servo\n",i);
			else if (model==MX106)
				printf("ID: %3d - MX-106 servo\n",i);
			else
				printf("ID: %3d - Unknown device with Model Number: %d\n",i,model);
		}
	}
}


void setChosenID()
{
	int toSet;

	//ask user for the ID
	printf("Input the ID of the device to operate [0,253]: ");
	scanf("%d",&toSet);
	
	if ( (toSet >= 0) && (toSet <= MAX_DYNAMIXEL) )
	{
		if (dxComm.dxl_ping(AX_SERIES,toSet))
		{
			chosenSeries = AX_SERIES;
		}else if (dxComm.dxl_ping(RX_SERIES,toSet))
		{
			chosenSeries = RX_SERIES;
		}
		else
		{
			printf("INPUT ID NOT DETECTED ON THE BUS! NO ACTION PERFORMED!\n");
			return;
		}

		chosenID = toSet;
		if (dxComm.dxl_read_word(chosenSeries,toSet,P_MODEL_NUMBER_L)==AX12)
		{
			chosenDevice = AX12;
			dxComm.dxl_write_word(chosenSeries,chosenID, P_GOAL_SPEED_L, 50);
		}
		else if (dxComm.dxl_read_word(chosenSeries,toSet,P_MODEL_NUMBER_L)==AX18)
		{
			chosenDevice = AX18;
//			dxComm.dxl_write_word(chosenSeries,chosenID, P_GOAL_SPEED_L, 50);
		}
		else if (dxComm.dxl_read_word(chosenSeries,toSet,P_MODEL_NUMBER_L)==RX28)
		{
			chosenDevice = RX28;
//			dxComm.dxl_write_word(chosenID, P_GOAL_SPEED_L, 50);
		}
		else if (dxComm.dxl_read_word(chosenSeries,toSet,P_MODEL_NUMBER_L)==RX64)
		{
			chosenDevice = RX64;
//			dxComm.dxl_write_word(chosenID, P_GOAL_SPEED_L, 50);
		}
		else if (dxComm.dxl_read_word(chosenSeries,toSet,P_MODEL_NUMBER_L)==MX106)
		{
			chosenDevice = MX106;
			chosenSeries = MX_SERIES;
//			dxComm.dxl_write_word(chosenID, P_GOAL_SPEED_L, 50);
		}
		else if (dxComm.dxl_read_word(chosenSeries,toSet,P_MODEL_NUMBER_L)==AXS1)
		{
			chosenDevice = AXS1;
		}
		else if (dxComm.dxl_read_word(chosenSeries,toSet,P_MODEL_NUMBER_L)==IMU6AXIS)
		{
			chosenDevice = IMU6AXIS;
		}
	}
	else
	{
		printf("INVALID INPUT ID! NO ACTION PERFORMED!\n");
	}
}


void setID()
{
	int ret,toSet, seriesToSet;

	//ask user for the series/channel to use for sending the packet
	printf("Input the SERIES of devices to use [0->TTL, 1->RS485]: ");
	ret = scanf("%d",&seriesToSet);

	//ask user for the ID
	printf("Input the ID you whish to write to the connected dynamixels [0,253]: ");
	ret = scanf("%d",&toSet);
	
	if ( (toSet >= 0) && (toSet <= MAX_DYNAMIXEL) )
		dxComm.dxl_write_byte(seriesToSet,BROADCAST_ID, P_ID, toSet);
	else
		printf("INVALID INPUT ID! NO ACTION PERFORMED!\n");
}


void setBaud()
{
	int ret,opt,toSet;

	printf("Choose the desired baudrate (in BPS):\n");
	printf("1 - 1000000\n");
	printf("2 - 500000\n");
	printf("3 - 400000\n");
	printf("4 - 250000\n");
	printf("5 - 200000\n");
	printf("6 - 115200\n");
	printf("7 - 57600\n");
	printf("8 - 19200\n");
	printf("9 - 9600\n");
	printf("0 - Cancel\n");
	printf("Choose your option: ");
	ret = scanf("%d",&opt);
	
	switch (opt)
	{
		case 1:
			toSet=1;
			break;
		case 2:
			toSet=3;
			break;
		case 3:
			toSet=4;
			break;
		case 4:
			toSet=7;
			break;
		case 5:
			toSet=9;
			break;
		case 6:
			toSet=16;
			break;
		case 7:
			toSet=34;
			break;
		case 8:
			toSet=103;
			break;
		case 9:
			toSet=207;
			break;
		default:
			return;
	}

		dxComm.dxl_write_byte(chosenSeries,BROADCAST_ID,P_BAUD_RATE,toSet);
}


void printMotorData()
{
	// EEPROM VALUES
	int baud, returnDelay, cwLimit, ccwLimit, highestTemp, lowestVolt, highestVolt, maxTorque, statusReturnLevel, alarmLED, alarmShutdown;
	// RAM VALUES
	int torque, led, cwComplianceMargin, ccwComplianceMargin, cwComplianceSlope, ccwComplianceSlope, goalPos, goalSpeed, torqueLim, presentPos, presentSpeed, presentLoad, presentVolt, presentTemp, registeredInst, moving, lock, punch;
	
	//control variables for continuous reading until key pressed
	char c;
	int n, tem;

	tem = fcntl(0, F_GETFL, 0);
	fcntl (0, F_SETFL, (tem | O_NDELAY));
	
	while (1)
	{
		// EEPROM general readings
		baud 				= dxComm.dxl_read_byte(chosenSeries,chosenID, P_BAUD_RATE);
		returnDelay			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_RETURN_DELAY_TIME);
		highestTemp			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_LIMIT_TEMPERATURE);
		lowestVolt			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_DOWN_LIMIT_VOLTAGE);
		highestVolt			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_UP_LIMIT_VOLTAGE);
		statusReturnLevel	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_RETURN_LEVEL);
		
		// EEPROM AX-12 only
		cwLimit				= dxComm.dxl_read_word(chosenSeries,chosenID, P_CW_ANGLE_LIMIT_L);
		ccwLimit			= dxComm.dxl_read_word(chosenSeries,chosenID, P_CCW_ANGLE_LIMIT_L);
		maxTorque			= dxComm.dxl_read_word(chosenSeries,chosenID, P_MAX_TORQUE_L);
		alarmLED			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_ALARM_LED);
		alarmShutdown		= dxComm.dxl_read_byte(chosenSeries,chosenID, P_ALARM_SHUTDOWN);

		// RAM general readings
		presentVolt			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_PRESENT_VOLTAGE);
		presentTemp			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_PRESENT_TEMPERATURE);
		registeredInst		= dxComm.dxl_read_byte(chosenSeries,chosenID, P_REGISTERED_INSTRUCTION);
		lock				= dxComm.dxl_read_byte(chosenSeries,chosenID, P_LOCK);
		
		// RAM AX-12 only
		torque				= dxComm.dxl_read_byte(chosenSeries,chosenID, P_TORQUE_ENABLE);
		led					= dxComm.dxl_read_byte(chosenSeries,chosenID, P_LED);
		cwComplianceMargin	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_CW_COMPLIANCE_MARGIN);
		ccwComplianceMargin	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_CCW_COMPLIANCE_MARGIN);
		cwComplianceSlope	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_CW_COMPLIANCE_SLOPE);
		ccwComplianceSlope	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_CCW_COMPLIANCE_SLOPE);
		goalPos				= dxComm.dxl_read_word(chosenSeries,chosenID, P_GOAL_POSITION_L);
		goalSpeed			= dxComm.dxl_read_word(chosenSeries,chosenID, P_GOAL_SPEED_L);
		torqueLim			= dxComm.dxl_read_word(chosenSeries,chosenID, P_TORQUE_LIMIT_L);
		presentPos			= dxComm.dxl_read_word(chosenSeries,chosenID, P_PRESENT_POSITION_L);
		presentSpeed		= dxComm.dxl_read_word(chosenSeries,chosenID, P_PRESENT_SPEED_L);
		presentLoad			= dxComm.dxl_read_word(chosenSeries,chosenID, P_PRESENT_LOAD_L);
		moving				= dxComm.dxl_read_byte(chosenSeries,chosenID, P_MOVING);
		punch				= dxComm.dxl_read_word(chosenSeries,chosenID, P_PUNCH_L);
		
		printf("\nCONTROL TABLE OF THE DEVICE WITH ID: %3d\n", chosenID);
		printf("------------------------------------------------\n");
		printf("| Address |           Item         |   Value   |\n");
		printf("------------------------------------------------\n");
		printf("|%2d (0x%2X)| Baud rate              |%4d(0x%3X)|\n", P_BAUD_RATE, P_BAUD_RATE, baud, baud);
		printf("|%2d (0x%2X)| Return delay time      |%4d(0x%3X)|\n", P_RETURN_DELAY_TIME, P_RETURN_DELAY_TIME, returnDelay, returnDelay);
		printf("|%2d (0x%2X)| CW angle limit         |%4d(0x%3X)|\n", P_CW_ANGLE_LIMIT_L, P_CW_ANGLE_LIMIT_L, cwLimit, cwLimit);
		printf("|%2d (0x%2X)| CCW angle limit        |%4d(0x%3X)|\n", P_CCW_ANGLE_LIMIT_L, P_CCW_ANGLE_LIMIT_L, ccwLimit, ccwLimit);
		printf("|%2d (0x%2X)| Limit temperature      |%4d(0x%3X)|\n", P_LIMIT_TEMPERATURE, P_LIMIT_TEMPERATURE, highestTemp, highestTemp);
		printf("|%2d (0x%2X)| Lowest limit voltage   |%4d(0x%3X)|\n", P_DOWN_LIMIT_VOLTAGE, P_DOWN_LIMIT_VOLTAGE, lowestVolt, lowestVolt);
		printf("|%2d (0x%2X)| Highet limit voltage   |%4d(0x%3X)|\n", P_UP_LIMIT_VOLTAGE, P_UP_LIMIT_VOLTAGE, highestVolt, highestVolt);
		printf("|%2d (0x%2X)| Max torque             |%4d(0x%3X)|\n", P_MAX_TORQUE_L, P_MAX_TORQUE_L, maxTorque, maxTorque);
		printf("|%2d (0x%2X)| Status return level    |%4d(0x%3X)|\n", P_RETURN_LEVEL, P_RETURN_LEVEL, statusReturnLevel, statusReturnLevel);
		printf("|%2d (0x%2X)| Alarm LED              |%4d(0x%3X)|\n", P_ALARM_LED, P_ALARM_LED, alarmLED, alarmLED);
		printf("|%2d (0x%2X)| Alarm shutdown         |%4d(0x%3X)|\n", P_ALARM_SHUTDOWN, P_ALARM_SHUTDOWN, alarmShutdown, alarmShutdown);
		printf("------------------------------------------------\n");
		printf("|%2d (0x%2X)| Torque enable          |%4d(0x%3X)|\n", P_TORQUE_ENABLE, P_TORQUE_ENABLE, torque, torque);
		printf("|%2d (0x%2X)| LED                    |%4d(0x%3X)|\n", P_LED, P_LED, led, led);
		printf("|%2d (0x%2X)| CW compliance margin   |%4d(0x%3X)|\n", P_CW_COMPLIANCE_MARGIN, P_CW_COMPLIANCE_MARGIN, cwComplianceMargin, cwComplianceMargin);
		printf("|%2d (0x%2X)| CCW compliance margin  |%4d(0x%3X)|\n", P_CCW_COMPLIANCE_MARGIN, P_CCW_COMPLIANCE_MARGIN, ccwComplianceMargin, ccwComplianceMargin);
		printf("|%2d (0x%2X)| CW compliance slope    |%4d(0x%3X)|\n", P_CW_COMPLIANCE_SLOPE, P_CW_COMPLIANCE_SLOPE, cwComplianceSlope, cwComplianceSlope);
		printf("|%2d (0x%2X)| CCW compliance slope   |%4d(0x%3X)|\n", P_CCW_COMPLIANCE_SLOPE, P_CCW_COMPLIANCE_SLOPE, ccwComplianceSlope, ccwComplianceSlope);
		
		printf("|%2d (0x%2X)| Goal position          |%4d(0x%3X)|\n", P_GOAL_POSITION_L, P_GOAL_POSITION_L, goalPos, goalPos);
		printf("|%2d (0x%2X)| Goal speed             |%4d(0x%3X)|\n", P_GOAL_SPEED_L, P_GOAL_SPEED_L, goalSpeed, goalSpeed);
		printf("|%2d (0x%2X)| Torque limit           |%4d(0x%3X)|\n", P_TORQUE_LIMIT_L, P_TORQUE_LIMIT_L, torqueLim, torqueLim);
		printf("|%2d (0x%2X)| Current position       |%4d(0x%3X)|\n", P_PRESENT_POSITION_L, P_PRESENT_POSITION_L, presentPos,presentPos);
		printf("|%2d (0x%2X)| Current speed          |%4d(0x%3X)|\n", P_PRESENT_SPEED_L, P_PRESENT_SPEED_L, presentSpeed, presentSpeed);
		printf("|%2d (0x%2X)| Current load           |%4d(0x%3X)|\n", P_PRESENT_LOAD_L, P_PRESENT_LOAD_L, presentLoad, presentLoad);
		printf("|%2d (0x%2X)| Current voltage        |%4d(0x%3X)|\n", P_PRESENT_VOLTAGE, P_PRESENT_VOLTAGE, presentVolt, presentVolt);
		printf("|%2d (0x%2X)| Current temperature    |%4d(0x%3X)|\n", P_PRESENT_TEMPERATURE, P_PRESENT_TEMPERATURE, presentTemp, presentTemp);
		printf("|%2d (0x%2X)| Registered instruction |%4d(0x%3X)|\n", P_REGISTERED_INSTRUCTION, P_REGISTERED_INSTRUCTION, registeredInst, registeredInst);
		printf("|%2d (0x%2X)| Moving                 |%4d(0x%3X)|\n", P_MOVING, P_MOVING, moving, moving);
		printf("|%2d (0x%2X)| Lock                   |%4d(0x%3X)|\n", P_LOCK, P_LOCK, lock, lock);
		printf("|%2d (0x%2X)| Punch                  |%4d(0x%3X)|\n", P_PUNCH_L, P_PUNCH_L, punch, punch);
		printf("------------------------------------------------\n");
		printf("ENTER TO TERMINATE CAPTURE CYCLE\n");
		
		n = read(0, &c, 1);
		if (n > 0) break;
	}

	fcntl(0, F_SETFL, tem);
}


void motorToPos()
{
	int ret,pos,speed;

	//ask user for the ID
	printf("Input the position to move the servo [0,1023]: ");
	ret = scanf("%d",&pos);
	
	printf("Input the speed to move the servo [0,1023]: ");
	ret = scanf("%d",&speed);
	
	if ( ((pos >= 0) && (pos <= 1023)) && ((speed >= 0) && (speed <= 1023)) )
	{
		dxComm.dxl_write_word(chosenSeries,chosenID, P_GOAL_POSITION_L, pos);
		dxComm.dxl_write_word(chosenSeries,chosenID, P_GOAL_SPEED_L, speed);
	}
	else
		printf("INVALID GOAL POSITION OR SPEED! NO ACTION PERFORMED!\n");
}


void motorAngleLimits()
{
	int ret,min,max;

	//ask user for the ID
	printf("Input the lower limit [0,1023]: ");
	ret = scanf("%d",&min);
	
	printf("Input the higher limit [0,1023] and >= lower limit: ");
	ret = scanf("%d",&max);
	
	if ( ((min >= 0) && (min <= 1023)) && ((max >= 0) && (max <= 1023)) && (min <= max) )
	{
		dxComm.dxl_write_word(chosenSeries,chosenID, P_CW_ANGLE_LIMIT_L, min);
		dxComm.dxl_write_word(chosenSeries,chosenID, P_CCW_ANGLE_LIMIT_L, max);
	}
	else
		printf("INVALID LIMITS! NO ACTION PERFORMED!\n");
}


void motorTorqueLimit()
{}


void motorDefaultValues()
{
	int position = dxComm.dxl_read_word(chosenSeries, chosenID, P_PRESENT_POSITION_L);
	
	dxComm.dxl_write_word(chosenSeries, chosenID, P_CW_ANGLE_LIMIT_L, 0);
	dxComm.dxl_write_word(chosenSeries, chosenID, P_CCW_ANGLE_LIMIT_L, 1023);
	dxComm.dxl_write_word(chosenSeries, chosenID, P_TORQUE_LIMIT_L, 1023);
	dxComm.dxl_write_byte(chosenSeries, chosenID, P_TORQUE_ENABLE, 0);
	dxComm.dxl_write_word(chosenSeries, chosenID, P_GOAL_SPEED_L, 0);
	dxComm.dxl_write_word(chosenSeries, chosenID, P_GOAL_POSITION_L, position );
}


void printImuData()
{
	// EEPROM VALUES
	int baud, returnDelay, statusReturnLevel;
	// RAM VALUES
	int led, sidewaysAccel, forwardsAccel, verticalAccel, pitch, roll, yaw;

	//control variables for continuous reading until key pressed
	char c;
	int n, tem;

	tem = fcntl(0, F_GETFL, 0);
	fcntl (0, F_SETFL, (tem | O_NDELAY));

	while (1)
	{
		baud 				= dxComm.dxl_read_byte(chosenSeries,chosenID, P_BAUD_RATE);
		returnDelay			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_RETURN_DELAY_TIME);
		statusReturnLevel	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_RETURN_LEVEL);

		led					= dxComm.dxl_read_byte(chosenSeries,chosenID, P_LED);
		sidewaysAccel		= dxComm.dxl_read_word(chosenSeries,chosenID, P_SIDEWAYS_ACCEL_L);
		forwardsAccel		= dxComm.dxl_read_word(chosenSeries,chosenID, P_FORWARDS_ACCEL_L);
		verticalAccel		= dxComm.dxl_read_word(chosenSeries,chosenID, P_VERTICAL_ACCEL_L);
		roll				= dxComm.dxl_read_word(chosenSeries,chosenID, P_ROLL_RATE_L);
		pitch				= dxComm.dxl_read_word(chosenSeries,chosenID, P_PITCH_RATE_L);
		yaw					= dxComm.dxl_read_word(chosenSeries,chosenID, P_YAW_RATE_L);

		printf("\nCONTROL TABLE OF THE DEVICE WITH ID: %3d\n", chosenID);
		printf("------------------------------------------------\n");
		printf("| Address |           Item         |   Value   |\n");
		printf("------------------------------------------------\n");
		printf("|%2d (0x%2X)| Baud rate              |%4d(0x%3X)|\n", P_BAUD_RATE, P_BAUD_RATE, baud, baud);
		printf("|%2d (0x%2X)| Return delay time      |%4d(0x%3X)|\n", P_RETURN_DELAY_TIME, P_RETURN_DELAY_TIME, returnDelay, returnDelay);
		printf("|%2d (0x%2X)| Status return level    |%4d(0x%3X)|\n", P_RETURN_LEVEL, P_RETURN_LEVEL, statusReturnLevel, statusReturnLevel);
		printf("|%2d (0x%2X)| LED                    |%4d(0x%3X)|\n", P_LED, P_LED, led, led);
		printf("|%2d (0x%2X)| Sideways Accel         |%4d(0x%3X)|\n", P_SIDEWAYS_ACCEL_L, P_SIDEWAYS_ACCEL_L, sidewaysAccel, sidewaysAccel);
		printf("|%2d (0x%2X)| Forwards Accel         |%4d(0x%3X)|\n", P_FORWARDS_ACCEL_L, P_FORWARDS_ACCEL_L, forwardsAccel, forwardsAccel);
		printf("|%2d (0x%2X)| Vertical Accel         |%4d(0x%3X)|\n", P_VERTICAL_ACCEL_L, P_VERTICAL_ACCEL_L, verticalAccel, verticalAccel);
		printf("|%2d (0x%2X)| Pitch rate             |%4d(0x%3X)|\n", P_PITCH_RATE_L, P_PITCH_RATE_L, pitch, pitch);
		printf("|%2d (0x%2X)| Roll rate              |%4d(0x%3X)|\n", P_ROLL_RATE_L, P_ROLL_RATE_L, roll, roll);
		printf("|%2d (0x%2X)| Yaw rate               |%4d(0x%3X)|\n", P_YAW_RATE_L, P_YAW_RATE_L, yaw, yaw);
		printf("------------------------------------------------\n");
		printf("ENTER TO TERMINATE CAPTURE CYCLE\n");
		
		n = read(0, &c, 1);
		if (n > 0) break;
	}

	fcntl(0, F_SETFL, tem);
}



void toggleLed()
{
	int led;
	
	led = dxComm.dxl_read_byte(chosenSeries, chosenID, P_LED);
	
	if ( led == 0 )
		dxComm.dxl_write_byte(chosenSeries, chosenID, P_LED, 1);
	else
		dxComm.dxl_write_byte(chosenSeries, chosenID, P_LED, 0);
}


void printAXs1Data()
{
		// EEPROM VALUES
	int baud, returnDelay, highestTemp, lowestVolt, highestVolt, statusReturnLevel, obstDetectThreshold, lightDetectThreshold;
	// RAM VALUES
	int leftIR, centerIR, rightIR, leftLight, centerLight, rightLight, obstacleDetectFlag, luminosityDetectFlag, soundData, soundDataMaxHold, soundDetectCount, soundDetectedTime, buzzerIndex, buzzerTime, presentVolt, presentTemp, registeredInst, irRemoconArrived, lock, irRemoconRxData0, irRemoconRxData1, irRemoconTxData0, irRemoconTxData1, obstacleDetectCompare, lightDetectCompare;
	
	//control variables for continuous reading until key pressed
	char c;
	int n, tem;

	tem = fcntl(0, F_GETFL, 0);
	fcntl (0, F_SETFL, (tem | O_NDELAY));
	
	while (1)
	{
		// EEPROM general readings
		baud 				= dxComm.dxl_read_byte(chosenSeries,chosenID, P_BAUD_RATE);
		returnDelay			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_RETURN_DELAY_TIME);
		highestTemp			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_LIMIT_TEMPERATURE);
		lowestVolt			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_DOWN_LIMIT_VOLTAGE);
		highestVolt			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_UP_LIMIT_VOLTAGE);
		statusReturnLevel	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_RETURN_LEVEL);
		
		// EEPROM AS-12 only
		obstDetectThreshold	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_EEPROM_IR_DETECT_COMPARE);
		lightDetectThreshold= dxComm.dxl_read_byte(chosenSeries,chosenID, P_EEPROM_LIGHT_DETECT_COMPARE);

		// RAM general readings
		presentVolt			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_PRESENT_VOLTAGE);
		presentTemp			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_PRESENT_TEMPERATURE);
		registeredInst		= dxComm.dxl_read_byte(chosenSeries,chosenID, P_REGISTERED_INSTRUCTION);
		lock				= dxComm.dxl_read_byte(chosenSeries,chosenID, P_LOCK);
		
		// RAM AS-12 only
		leftIR				= dxComm.dxl_read_byte(chosenSeries,chosenID, P_IR_LEFT_FIRE_DATA);
		centerIR			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_IR_CENTER_FIRE_DATA);
		rightIR				= dxComm.dxl_read_byte(chosenSeries,chosenID, P_IR_RIGHT_FIRE_DATA);
		leftLight			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_LIGHT_LEFT_DATA);
		centerLight			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_LIGHT_CENTER_DATA);
		rightLight			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_LIGHT_RIGHT_DATA);
		obstacleDetectFlag	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_IR_OBSTACLE_DETECTED);
		luminosityDetectFlag= dxComm.dxl_read_byte(chosenSeries,chosenID, P_LIGHT_DETECTED);
		soundData			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_SOUND_DATA);
		soundDataMaxHold	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_SOUND_DATA_MAX_HOLD);
		soundDetectCount	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_SOUND_DETECTED_COUNT);
		soundDetectedTime	= dxComm.dxl_read_word(chosenSeries,chosenID, P_SOUND_DETECTED_TIME_L);
		buzzerIndex			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_BUZZER_DATA0);
		buzzerTime			= dxComm.dxl_read_byte(chosenSeries,chosenID, P_BUZZER_DATA1);
		irRemoconArrived	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_REMOCON_ARRIVED);
		irRemoconRxData0	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_REMOCON_RXDATA0);
		irRemoconRxData1	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_REMOCON_RXDATA1);
		irRemoconTxData0	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_REMOCON_TXDATA0);
		irRemoconTxData1	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_REMOCON_TXDATA1);
		obstacleDetectCompare= dxComm.dxl_read_byte(chosenSeries,chosenID, P_IR_DETECT_COMPARE);
		lightDetectCompare	= dxComm.dxl_read_byte(chosenSeries,chosenID, P_LIGHT_DETECT_COMPARE);

		printf("\nCONTROL TABLE OF THE DEVICE WITH ID: %3d\n", chosenID);
		printf("-------------------------------------------------------\n");
		printf("| Address |              Item             |   Value   |\n");
		printf("-------------------------------------------------------\n");
		printf("|%2d (0x%2X)| Baud rate                     |%4d(0x%3X)|\n", P_BAUD_RATE, P_BAUD_RATE, baud, baud);
		printf("|%2d (0x%2X)| Return delay time             |%4d(0x%3X)|\n", P_RETURN_DELAY_TIME, P_RETURN_DELAY_TIME, returnDelay, returnDelay);
		printf("|%2d (0x%2X)| Limit temperature             |%4d(0x%3X)|\n", P_LIMIT_TEMPERATURE, P_LIMIT_TEMPERATURE, highestTemp, highestTemp);
		printf("|%2d (0x%2X)| Lowest limit voltage          |%4d(0x%3X)|\n", P_DOWN_LIMIT_VOLTAGE, P_DOWN_LIMIT_VOLTAGE, lowestVolt, lowestVolt);
		printf("|%2d (0x%2X)| Highet limit voltage          |%4d(0x%3X)|\n", P_UP_LIMIT_VOLTAGE, P_UP_LIMIT_VOLTAGE, highestVolt, highestVolt);
		printf("|%2d (0x%2X)| Status return level           |%4d(0x%3X)|\n", P_RETURN_LEVEL, P_RETURN_LEVEL, statusReturnLevel, statusReturnLevel);
		printf("|%2d (0x%2X)| Obstacle detect compare value |%4d(0x%3X)|\n", P_EEPROM_IR_DETECT_COMPARE, P_EEPROM_IR_DETECT_COMPARE, obstDetectThreshold, obstDetectThreshold);
		printf("|%2d (0x%2X)| Light detect compare value    |%4d(0x%3X)|\n", P_EEPROM_LIGHT_DETECT_COMPARE, P_EEPROM_LIGHT_DETECT_COMPARE, lightDetectThreshold, lightDetectThreshold);
		printf("-------------------------------------------------------\n");
		printf("|%2d (0x%2X)| Left IR sensor data           |%4d(0x%3X)|\n", P_IR_LEFT_FIRE_DATA, P_IR_LEFT_FIRE_DATA, leftIR, leftIR);
		printf("|%2d (0x%2X)| Center IR sensor data         |%4d(0x%3X)|\n", P_IR_CENTER_FIRE_DATA, P_IR_CENTER_FIRE_DATA, centerIR, centerIR);
		printf("|%2d (0x%2X)| Right IR sensor data          |%4d(0x%3X)|\n", P_IR_RIGHT_FIRE_DATA, P_IR_RIGHT_FIRE_DATA, rightIR, rightIR);
		printf("|%2d (0x%2X)| Left luminosity               |%4d(0x%3X)|\n", P_LIGHT_LEFT_DATA, P_LIGHT_LEFT_DATA, leftLight, leftLight);
		printf("|%2d (0x%2X)| Center luminosity             |%4d(0x%3X)|\n", P_LIGHT_CENTER_DATA, P_LIGHT_CENTER_DATA, centerLight, centerLight);
		printf("|%2d (0x%2X)| Right luminosity              |%4d(0x%3X)|\n", P_LIGHT_RIGHT_DATA, P_LIGHT_RIGHT_DATA, rightLight, rightLight);
		printf("|%2d (0x%2X)| Obstacle detection flag       |%4d(0x%3X)|\n", P_IR_OBSTACLE_DETECTED, P_IR_OBSTACLE_DETECTED, obstacleDetectFlag, obstacleDetectFlag);
		printf("|%2d (0x%2X)| Luminosity detection flag     |%4d(0x%3X)|\n", P_LIGHT_DETECTED, P_LIGHT_DETECTED, luminosityDetectFlag, luminosityDetectFlag);
		printf("|%2d (0x%2X)| Sound data                    |%4d(0x%3X)|\n", P_SOUND_DATA, P_SOUND_DATA, soundData, soundData);
		printf("|%2d (0x%2X)| Sound data max hold           |%4d(0x%3X)|\n", P_SOUND_DATA_MAX_HOLD, P_SOUND_DATA_MAX_HOLD, soundDataMaxHold, soundDataMaxHold);
		printf("|%2d (0x%2X)| Sound detected count          |%4d(0x%3X)|\n", P_SOUND_DETECTED_COUNT, P_SOUND_DETECTED_COUNT, soundDetectCount, soundDetectCount);
		printf("|%2d (0x%2X)| Sound detected time           |%4d(0x%3X)|\n", P_SOUND_DETECTED_TIME_L, P_SOUND_DETECTED_TIME_L, soundDetectedTime, soundDetectedTime);
		printf("|%2d (0x%2X)| Buzzer index                  |%4d(0x%3X)|\n", P_BUZZER_DATA0, P_BUZZER_DATA0, buzzerIndex, buzzerIndex);
		printf("|%2d (0x%2X)| Buzzer time                   |%4d(0x%3X)|\n", P_BUZZER_DATA1, P_BUZZER_DATA1, buzzerTime, buzzerTime);
		printf("|%2d (0x%2X)| Current voltage               |%4d(0x%3X)|\n", P_PRESENT_VOLTAGE, P_PRESENT_VOLTAGE, presentVolt, presentVolt);
		printf("|%2d (0x%2X)| Current temperature           |%4d(0x%3X)|\n", P_PRESENT_TEMPERATURE, P_PRESENT_TEMPERATURE, presentTemp, presentTemp);
		printf("|%2d (0x%2X)| Registered instruction        |%4d(0x%3X)|\n", P_REGISTERED_INSTRUCTION, P_REGISTERED_INSTRUCTION, registeredInst, registeredInst);
		printf("|%2d (0x%2X)| IR remocon arrived            |%4d(0x%3X)|\n", P_REMOCON_ARRIVED, P_REMOCON_ARRIVED, irRemoconArrived, irRemoconArrived);
		printf("|%2d (0x%2X)| Lock                          |%4d(0x%3X)|\n", P_LOCK, P_LOCK, lock, lock);
		printf("|%2d (0x%2X)| IR remocon RX data 0          |%4d(0x%3X)|\n", P_REMOCON_RXDATA0, P_REMOCON_RXDATA0, irRemoconRxData0, irRemoconRxData0);
		printf("|%2d (0x%2X)| IR remocon RX data 1          |%4d(0x%3X)|\n", P_REMOCON_RXDATA1, P_REMOCON_RXDATA1, irRemoconRxData1, irRemoconRxData1);
		printf("|%2d (0x%2X)| IR remocon TX data 0          |%4d(0x%3X)|\n", P_REMOCON_TXDATA0, P_REMOCON_TXDATA0, irRemoconTxData0, irRemoconTxData0);
		printf("|%2d (0x%2X)| IR remocon TX data 1          |%4d(0x%3X)|\n", P_REMOCON_TXDATA1, P_REMOCON_TXDATA1, irRemoconTxData1, irRemoconTxData1);
		printf("|%2d (0x%2X)| Obstacle detect compare       |%4d(0x%3X)|\n", P_IR_DETECT_COMPARE, P_IR_DETECT_COMPARE, obstacleDetectCompare, obstacleDetectCompare);
		printf("|%2d (0x%2X)| Light detect compare          |%4d(0x%3X)|\n", P_LIGHT_DETECT_COMPARE, P_LIGHT_DETECT_COMPARE, lightDetectCompare, lightDetectCompare);
		printf("-------------------------------------------------------\n");
		printf("ENTER TO TERMINATE CAPTURE CYCLE\n");
		
		n = read(0, &c, 1);
		if (n > 0) break;
	}

	fcntl(0, F_SETFL, tem);
}



///////////////////////////////////////////////
///////// AUXILIARY STATUS FUNCTIONS //////////
///////////////////////////////////////////////

// Function to be called after each action so the menu doesn't pop up again and takes the info of the screen (much)
void backToMainMenu()
{
	int dummy;
	
	printf("Press Enter key to go back to Main Menu...\n");
	getchar();
	getchar();
	dummy = ::system("clear");
}


//// Print communication result
//void PrintCommStatus(int CommStatus)
//{
//	switch(CommStatus)
//	{
//	case COMM_TXFAIL:
//		printf("COMM_TXFAIL: Failed transmit instruction packet!\n");
//		break;
//
//	case COMM_TXERROR:
//		printf("COMM_TXERROR: Incorrect instruction packet!\n");
//		break;
//
//	case COMM_RXFAIL:
//		printf("COMM_RXFAIL: Failed get status packet from device!\n");
//		break;
//
//	case COMM_RXWAITING:
//		printf("COMM_RXWAITING: Now recieving status packet!\n");
//		break;
//
//	case COMM_RXTIMEOUT:
//		printf("COMM_RXTIMEOUT: There is no status packet!\n");
//		break;
//
//	case COMM_RXCORRUPT:
//		printf("COMM_RXCORRUPT: Incorrect status packet!\n");
//		break;
//
//	default:
//		printf("This is unknown error code!\n");
//		break;
//	}
//}

//// Print error bit of status packet
//void PrintErrorCode()
//{
//	if(dxl_get_rxpacket_error(ERRBIT_VOLTAGE) == 1)
//		printf("Input voltage error!\n");
//
//	if(dxl_get_rxpacket_error(ERRBIT_ANGLE) == 1)
//		printf("Angle limit error!\n");
//
//	if(dxl_get_rxpacket_error(ERRBIT_OVERHEAT) == 1)
//		printf("Overheat error!\n");
//
//	if(dxl_get_rxpacket_error(ERRBIT_RANGE) == 1)
//		printf("Out of range error!\n");
//
//	if(dxl_get_rxpacket_error(ERRBIT_CHECKSUM) == 1)
//		printf("Checksum error!\n");
//
//	if(dxl_get_rxpacket_error(ERRBIT_OVERLOAD) == 1)
//		printf("Overload error!\n");
//
//	if(dxl_get_rxpacket_error(ERRBIT_INSTRUCTION) == 1)
//		printf("Instruction code error!\n");
//}
